#include "xplodeconfig.h"

// Shared variables
QString *plasmPath;
QString *plasmVrmlViewer;
QString *plasmSvgViewer;
QString *plasmFlashViewer;
QFont   *plasmEditorFont;
QFont   *plasmListenerFont;
QString *plasmNewLine;

// Constructor
XplodeConfig::XplodeConfig()
{
	// Construct the configuration file path for the current operating system
#ifdef linux
	// Get the user home directory
	configFile = new QString(getenv("HOME"));
	// Build the configuration file path
	configFile->append("/.xploderc.xml");
#endif

#ifdef WIN32
	// Get the user home directory
	configFile = new QString(getenv("USERPROFILE"));
	// If "USERPROFILE" is not available, then we're running Windows 9x/Me
	if (configFile->isEmpty()) *configFile = "C:\\WINDOWS";
	configFile->append("\\xploderc.xml");
#endif

#ifdef __APPLE__
	// Get the user home directory
	configFile = new QString(getenv("HOME"));
	// Build the configuration file path
	configFile->append("/.xploderc.xml");
#endif

	// Set the proper carriage return ('\n' on unix systems, '\n\r' on windows systems)
	plasmNewLine = new QString();

#ifdef linux
	*plasmNewLine = "\n";
#endif

#ifdef WIN32
	*plasmNewLine = "\n\r";
#endif

#ifdef __APPLE__
	*plasmNewLine = "\n";
#endif
}

// Destructor
XplodeConfig::~XplodeConfig()
{
	delete configFile;
}

// Read the configuration file and set the shared variables
bool XplodeConfig::readConfig()
{
	// Open the configuration file
	QFile config(*configFile);
	// If an error occurs, exit
	if (!config.open(IO_ReadOnly)) return false;
	// Build the XML document
	configDoc = new QDomDocument();
	// Parse the XML configuration. On error, exit
	if (!configDoc->setContent(&config)) return false;
	// Close the file
	config.close();
	// Create the root element and a dummy one, used for parsing
	QDomElement root, elem;
	root = configDoc->documentElement();
	elem = root.firstChild().toElement();
	// Check the version of configuration file. On error exit
	if (root.attribute("version") != "2.0") return false;
	QString tmp;
	// Loop until the end of document
	while (!elem.isNull())
	{
		// <plasm-path> tag
		if (elem.tagName() == "plasm-path")
		{
			// Check all attributes. If one is missing, exit
			if (!elem.attribute("path").isEmpty())
				plasmPath = new QString(elem.attribute("path"));
			else
				return false;
		}
		// <vrml-viewer> tag
		if (elem.tagName() == "vrml-viewer")
		{
			// Check all attributes. If one is missing, exit
			if (!elem.attribute("path").isEmpty())
				plasmVrmlViewer = new QString(elem.attribute("path"));
			else
				return false;
		}
		// <svg-viewer> tag
		if (elem.tagName() == "svg-viewer")
		{
			// Check all attributes. If one is missing, exit
			if (!elem.attribute("path").isEmpty())
				plasmSvgViewer = new QString(elem.attribute("path"));
			else
				return false;
		}
		// <flash-viewer> tag
		if (elem.tagName() == "flash-viewer")
		{
			// Check all attributes. If one is missing, exit
			if (!elem.attribute("path").isEmpty())
				plasmFlashViewer = new QString(elem.attribute("path"));
			else
				return false;
		}
		// <editor-font> tag
		if (elem.tagName() == "editor-font")
		{
			QString fontName = elem.attribute("name"),
					fontSize = elem.attribute("size"),
					fontWeight = elem.attribute("weight");
			// Check all attributes. If one is missing, exit
			if (fontName.isEmpty() || fontSize.isEmpty() || fontWeight.isEmpty()) return false;
			// Font size and weight are positive integers
			if (fontSize.toInt() <= 0) return false;
			if (fontWeight.toInt() <= 0) return false;
			plasmEditorFont = new QFont(fontName, fontSize.toInt(), fontWeight.toInt());
		}
		// <listener-font> tag
		if (elem.tagName() == "listener-font")
		{
			QString fontName = elem.attribute("name"),
					fontSize = elem.attribute("size"),
					fontWeight = elem.attribute("weight");
			// Check all attributes. If one is missing, exit
			if (fontName.isEmpty() || fontSize.isEmpty() || fontWeight.isEmpty()) return false;
			// Font size and weight are positive integers
			if (fontSize.toInt() <= 0) return false;
			if (fontWeight.toInt() <= 0) return false;
			plasmListenerFont = new QFont(fontName, fontSize.toInt(), fontWeight.toInt());
		}
		elem = elem.nextSibling().toElement();
	}
	// Now all shared variables should be set to non-null. If not, exit
	if (!plasmEditorFont ||
		!plasmListenerFont ||
		!plasmNewLine ||
		!plasmPath ||
		!plasmSvgViewer ||
		!plasmFlashViewer ||
		!plasmVrmlViewer) return false;
	return true;
}

// Save the configuration file
bool XplodeConfig::saveConfig()
{
	// Open the configuration file and connect it to a text stream
	QFile config(*configFile);
	QTextStream stream(&config);
	// If an error occurs, exit
	if (!config.open(IO_WriteOnly)) return false;
	stream << "<?xml version='1.0' standalone='yes' ?>" << *plasmNewLine;
	stream << *plasmNewLine;
	stream << "<xplode-config version='2.0'>" << *plasmNewLine;
	stream << "    <plasm-path path='" << *plasmPath << "' />" << *plasmNewLine;
	stream << "    <vrml-viewer path='" << *plasmVrmlViewer << "' />" << *plasmNewLine;
	stream << "    <svg-viewer path='" << *plasmSvgViewer << "' />" << *plasmNewLine;
	stream << "    <flash-viewer path='" << *plasmFlashViewer << "' />" << *plasmNewLine;
	stream << "    <editor-font name='" << plasmEditorFont->family() << "' size='" <<
			plasmEditorFont->pointSize() << "' weight='" <<
			plasmEditorFont->weight() << "' />" << *plasmNewLine;
	stream << "    <listener-font name='" << plasmListenerFont->family() << "' size='" <<
			plasmListenerFont->pointSize() << "' weight='" <<
			plasmListenerFont->weight() << "' />" << *plasmNewLine;
	stream << "</xplode-config>";
	config.close();
	return true;
}

// Creates a standard configuration file
bool XplodeConfig::rebuildConfig()
{
	// Open the configuration file and connect it to a text stream
	QFile config(*configFile);
	QTextStream stream(&config);
	// If an error occurs, exit
	if (!config.open(IO_WriteOnly)) return false;
	stream << "<?xml version='1.0' standalone='yes' ?>" << *plasmNewLine;
	stream << *plasmNewLine;
	stream << "<xplode-config version='2.0'>" << *plasmNewLine;

#ifdef linux
	stream << "    <plasm-path path='/usr/local/plasm/plasm' />" << *plasmNewLine;
	stream << "    <vrml-viewer path='/usr/local/plasm/vrmlview' />" << *plasmNewLine;
	stream << "    <svg-viewer path='/usr/local/plasm/svgview' />" << *plasmNewLine;
	stream << "    <flash-viewer path='/usr/local/plasm/flashview' />" << *plasmNewLine;
	stream << "    <editor-font name='courier' size='12' weight='40' />" << *plasmNewLine;
	stream << "    <listener-font name='courier' size='12' weight='40' />" << *plasmNewLine;
#endif

#ifdef __APPLE__
	stream << "    <plasm-path path='/Applications/plasm/plasm' />" << *plasmNewLine;
	stream << "    <vrml-viewer path='/Applications/Safari.app' />" << *plasmNewLine;
	stream << "    <svg-viewer path='/Applications/Safari.app' />" << *plasmNewLine;
	stream << "    <flash-viewer path='/Applications/Safari.app' />" << *plasmNewLine;
	stream << "    <editor-font name='courier' size='12' weight='40' />" << *plasmNewLine;
	stream << "    <listener-font name='courier' size='12' weight='40' />" << *plasmNewLine;
#endif

#ifdef WIN32
	// Get Xplode path
	QDir d(*plasmXplode);
	QString absPath = d.absPath();                     
	// Get the directory
	QString path = absPath.left(absPath.length() - 10);
	stream << "    <plasm-path path='" << path << "plasm.exe' />" << *plasmNewLine;
	stream << "    <vrml-viewer path='" << path << "vrmlview.exe' />" << *plasmNewLine;
	stream << "    <svg-viewer path='" << path << "svgview.exe' />" << *plasmNewLine;
	stream << "    <flash-viewer path='" << path << "flashview.exe' />" << *plasmNewLine;
	stream << "    <editor-font name='courier new' size='12' weight='40' />" << *plasmNewLine;
	stream << "    <listener-font name='courier new' size='12' weight='40' />" << *plasmNewLine;
#endif

	stream << "</xplode-config>";
	config.close();
	return true;
}
