#include "xplodelistener.h"
#include "xplodeshared.h"

// Constructor
XplodeListener::XplodeListener(QWidget *parent) : QTextEdit(parent)
{
	// Window title
	setCaption("Xplode Listener");
	// No connection is up
	connected  = false;
	// Set up the listener font
	setFont(*plasmListenerFont);
	// No modification allowed
	setReadOnly(true);
	// Get the palette
	QPalette palette = this->palette();
	// Set black background for the active window
	palette.setBrush(QPalette::Active, QColorGroup::Background, QColor(0, 0, 0));
	palette.setBrush(QPalette::Active, QColorGroup::Base, QColor(0, 0, 0));
	// Set black background for the inactive window
	palette.setBrush(QPalette::Inactive, QColorGroup::Background, QColor(0, 0, 0));
	palette.setBrush(QPalette::Inactive, QColorGroup::Base, QColor(0, 0, 0));
	setPalette(palette);
	// No undo
	setUndoDepth(0);
	// No MzPlasm process running
	mzplasm = 0;
}

// Destructor
XplodeListener::~XplodeListener()
{
}

void XplodeListener::updateFont()
{
	// Set font and repaint
	setFont(*plasmListenerFont);
	repaintContents(true);
}

// Connect to Plasm interpreter
void XplodeListener::openConnection()
{
	// If we have a MzPlasm process, something is wrong
	if (mzplasm)
	{
		QMessageBox::critical(this, "Xplode Listener Error", "A Plasm process is already running." \
									"\nSomething is really wrong.");
		return;
	}
	// Execute 'Plasm_Executable --dir Plasm_Path
	mzplasm = new QProcess(this);
	// Build MzPlasm_Executable
	mzplasm->addArgument(*plasmPath);
	mzplasm->addArgument("--dir");
	// Build MzPlasm_Path
	mzplasm->addArgument(QFileInfo(*plasmPath).dirPath(true));
	// Start the process, exit on error
	if (!mzplasm->start())
	{
		QMessageBox::critical(this, "Xplode Listener Error", "Cannot start the Plasm process.\nCheck your settings.");
		// Evaluation stopped
		emit stopEvaluation();
		return;
	}
	// Wait until MzPlasm produces an output
	connect(mzplasm, SIGNAL(readyReadStdout()), this, SLOT(receive()));
}

// Reconnect to the interpreter, closing last connection
void XplodeListener::reconnect()
{
	// Close previous connection
	closeConnection();
	// Re-connect
	openConnection();
}

void XplodeListener::closeConnection()
{
	// If we're still connected
	if (connected)
	{
		// Send the server the quit message
		QString exitMsg = "(exit);" + *plasmNewLine;
		mzplasm->writeToStdin(exitMsg);
	}
	// If MzPlasm is still running (this should not happen)
	if (mzplasm)
	{
		// Disconnect signals
		disconnect(mzplasm, SIGNAL(readyReadStdout()), this, SLOT(receive()));
		// Kill the process
		mzplasm->kill();
		delete mzplasm;
		mzplasm = 0;
	}
	connected = false;
	// Clear the listener
	clear();
	emit stopEvaluation();
}

// Receive data from socket and handle it
void XplodeListener::receive()
{
	// Now the connection is estabilished
	if (!connected) connected = true;
	// Read MzPlasm output (stdout)
	QString buffer = mzplasm->readStdout();
	// Append the output to the console
	setColor(QColor(230, 230, 230));
	append(buffer);
	// If MzPlasm ends evaluating (shows the prompt) stop the animation
	if (buffer.find("plasm> ") != -1) emit stopEvaluation();
	// Find exported files
	int index = buffer.find("Filename = ");
	while (index != -1)
	{
		// Get the file name
		QString fileName = buffer.mid(buffer.find("=", index) + 2, buffer.find("\n", index) - buffer.find("=", index) - 2);
		// Signal there's a new exported object
		emit exported(fileName);
		// Find the next exported file
		index = buffer.find("Filename = ", buffer.find("\n", index));
	}
	// If there are more than 5000 lines, delete the first 4000
	if (paragraphs() > 5000)
	{
		qWarning("Purging");
		setSelection(0, 0, 4000, text(4000).length());
		removeSelectedText();
		// Move the cursor to the end
		scrollToBottom();
	}
}

// Tell MzPlasm to evaluate some code
void XplodeListener::eval(QString code)
{
	// If not connected, exit
	if (!connected) return;
	// Add the scheme function '(plasm "'
	code.prepend("(plasm \"");
	// Append the ending '");'
	code.append("\");" + *plasmNewLine);
	// Send the code to the MzPlasm server
	mzplasm->writeToStdin(code);
}

bool XplodeListener::isConnected()
{
	return connected;
}
